/*
 * DBLoggerApp.cpp -- David Evans, December 28th, 1996
 */

#include <Volume.h>
#include <Table.h>
#include <Record.h>
#include <SupportDefs.h>
#include <Alert.h>
#include <stdio.h>
#include "DBLoggerApp.h"

DBLoggerApp::DBLoggerApp()
	: BApplication('logr')
{}

void DBLoggerApp::ReadyToRun(void)
{
	BVolume vol = boot_volume();
	db = vol.Database();

	db->Sync();
	if((logTable = db->FindTable("Log Message")) == NULL)
	{
		// Must create the log message table.
		//
		// Should make the error checking better.
		
		logTable = db->CreateTable("Log Message", "BrowserItem");
		logTable->AddStringField("Message");
		logTable->AddStringField("Severity", B_INDEXED_FIELD);
		db->Sync();
	}
	
	nameKey = logTable->FieldKey("Name");
	createdKey = logTable->FieldKey("Created");
	modifiedKey = logTable->FieldKey("Modified");
	messageKey = logTable->FieldKey("Message");
	severityKey = logTable->FieldKey("Severity");
	
}

DBLoggerApp::~DBLoggerApp()
{}

void DBLoggerApp::MessageReceived(BMessage *message)
{
	if(message->what == D_LOG_MESSAGE)
	{
		const char *logMessage, *severity;
		char *name;
		double time;
		
		if((logMessage = message->FindString("Log Message")) == NULL)
			return;
		
		if((severity = message->FindString("Severity")) == NULL)
			return;
		
		if(message->FindString("Name") == NULL)
		{
			BMessenger remoteSender = message->ReturnAddress();
			if(message->Error() != B_NO_ERROR)
				return;
			
			team_id remoteTeam = remoteSender.Team();
			app_info remoteAppInfo;
			if(be_roster->GetRunningAppInfo(remoteTeam, &remoteAppInfo) != B_NO_ERROR)
				return;
	
			thread_id remoteMainThread = remoteAppInfo.thread;
			thread_info remoteThreadInfo;
			if(get_thread_info(remoteMainThread, &remoteThreadInfo) != B_NO_ERROR)
				return;
			
			name = (char *)malloc(strlen(remoteThreadInfo.name) + 1);
			strcpy(name, remoteThreadInfo.name);
		}
		else
		{
			name = (char *)malloc(strlen(message->FindString("Name")) + 1);
			strcpy(name, message->FindString("Name"));
		}

		BRecord *logRecord = new BRecord(logTable);
		logRecord->SetString(nameKey, name);
		
		time = real_time_clock();
		logRecord->SetTime(createdKey, time);
		logRecord->SetTime(modifiedKey, time);
		logRecord->SetString(messageKey, logMessage);
		logRecord->SetString(severityKey, severity);
		
		logRecord->Commit();
		
		delete logRecord;
		free(name);
	}
	else
		inherited::MessageReceived(message);
		
}
